"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var server_vars_1 = require("../test/server-vars");
var config_1 = require("./config");
describe('Config', function () {
    beforeEach(function () {
        server_vars_1.clearConfig();
    });
    describe('Production', function () {
        test('throws if required values are not specified', function () { return __awaiter(void 0, void 0, void 0, function () {
            return __generator(this, function (_a) {
                process.env.NODE_ENV = 'production';
                try {
                    new config_1.Config({ configSearchPath: __dirname });
                }
                catch (error) {
                    expect(error.message).toContain('WARTHOG_DB_HOST is required');
                }
                return [2 /*return*/];
            });
        }); });
    });
    describe('Development', function () {
        test('uses correct defaults', function () { return __awaiter(void 0, void 0, void 0, function () {
            var config;
            return __generator(this, function (_a) {
                process.env.NODE_ENV = 'development';
                config = new config_1.Config({ configSearchPath: __dirname });
                expect(config.get('DB_HOST')).toEqual('localhost');
                return [2 /*return*/];
            });
        }); });
    });
    describe('Test', function () {
        test('will never open playground', function () { return __awaiter(void 0, void 0, void 0, function () {
            var config;
            return __generator(this, function (_a) {
                process.env = {
                    NODE_ENV: 'development',
                    WARTHOG_AUTO_OPEN_PLAYGROUND: 'true',
                    JEST_WORKER_ID: '12345'
                };
                config = new config_1.Config({ configSearchPath: __dirname });
                expect(config.get('WARTHOG_AUTO_OPEN_PLAYGROUND')).toEqual('false');
                return [2 /*return*/];
            });
        }); });
    });
    describe('All environments', function () {
        test('translates TYPEORM env vars into warthog config', function () { return __awaiter(void 0, void 0, void 0, function () {
            var config;
            return __generator(this, function (_a) {
                process.env = {
                    NODE_ENV: 'development',
                    TYPEORM_FOO: 'baz456'
                };
                config = new config_1.Config({ configSearchPath: __dirname });
                expect(config.get('WARTHOG_DB_FOO')).toEqual('baz456');
                return [2 /*return*/];
            });
        }); });
        test('uses WARTHOG_DB_URL if filled in', function () { return __awaiter(void 0, void 0, void 0, function () {
            var config;
            return __generator(this, function (_a) {
                process.env = {
                    NODE_ENV: 'development',
                    WARTHOG_DB_USERNAME: 'foo',
                    WARTHOG_DB_PASSWORD: 'bar',
                    WARTHOG_DB_URL: 'postgres://user123:password456@server-one-two.foo.provider.com:8888/database_name'
                };
                config = new config_1.Config({ configSearchPath: __dirname });
                expect(config.get('WARTHOG_DB_USERNAME')).toEqual('user123');
                expect(config.get('WARTHOG_DB_PASSWORD')).toEqual('password456');
                expect(config.get('WARTHOG_DB_HOST')).toEqual('server-one-two.foo.provider.com');
                expect(config.get('WARTHOG_DB_PORT')).toEqual('8888');
                expect(config.get('WARTHOG_DB_DATABASE')).toEqual('database_name');
                return [2 /*return*/];
            });
        }); });
    });
});
//# sourceMappingURL=config.test.js.map