"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMetadataStorage = exports.MetadataStorage = void 0;
var typedi_1 = require("typedi");
// TODO: we need to ensure there is an ID field on the model
// MetadataStorage class is only responsible for capturing raw metadata, not additional business logic or other defaults
// Consumers should work off of this raw data
var MetadataStorage = /** @class */ (function () {
    function MetadataStorage() {
        this.enumMap = {};
        this.classMap = {};
        this.models = {};
        this.interfaces = [];
    }
    MetadataStorage.prototype.addModel = function (name, klass, filename, options) {
        if (options === void 0) { options = {}; }
        if (this.interfaces.indexOf(name) > -1) {
            return; // Don't add interface types to model list
        }
        this.classMap[name] = {
            filename: filename,
            klass: klass,
            name: name
        };
        // We shouldn't need to do this as a field decorator will almost certainly have called this
        this.ensureModelExists(name, klass);
        // Just add `klass` and `filename` to the model object
        this.models[name] = __assign(__assign(__assign({}, this.models[name]), { filename: filename }), options);
        // Only add klass if it wasn't already added by `addField`
        if (!this.models[name].klass) {
            this.models[name].klass = klass;
        }
    };
    MetadataStorage.prototype.addEnum = function (target, columnName, enumName, enumValues, filename, options) {
        var modelName = target.name;
        this.enumMap[modelName] = this.enumMap[modelName] || {};
        this.enumMap[modelName][columnName] = {
            enumeration: enumValues,
            filename: filename,
            name: enumName
        };
        // the enum needs to be passed so that it can be bound to column metadata
        options.enum = enumValues;
        options.enumName = enumName;
        this.addField('enum', target, columnName, options);
    };
    MetadataStorage.prototype.getModels = function () {
        return this.models;
    };
    // https://stackoverflow.com/questions/42281045/can-typescript-property-decorators-set-metadata-for-the-class
    // https://stackoverflow.com/questions/55117125/typescript-decorators-reflect-metadata/55117327#55117327
    MetadataStorage.prototype.collect = function () {
        var _this = this;
        Object.keys(this.models).forEach(function (modelName) {
            var target = _this.models[modelName].klass;
            var keys = Reflect.getMetadataKeys(target);
            keys.forEach(function (item) {
                if (!item.startsWith('warthog:field')) {
                    return;
                }
                var metadata = Reflect.getMetadata(item, target);
                if (metadata.type === 'enum') {
                    _this.addEnum(target, metadata.propertyKey, metadata.name, metadata.enumeration, metadata.relativeFilePath, metadata.options);
                }
                else {
                    _this.addField(metadata.type, target, metadata.propertyKey, metadata.options);
                }
            });
        });
        return this;
    };
    MetadataStorage.prototype.getModel = function (name) {
        return this.models[name];
    };
    MetadataStorage.prototype.getModelColumns = function (modelName) {
        var model = this.getModel(modelName);
        return model.columns.map(function (column) { return column.propertyName; });
    };
    MetadataStorage.prototype.getEnum = function (modelName, columnName) {
        if (!this.enumMap[modelName]) {
            return undefined;
        }
        return this.enumMap[modelName][columnName] || undefined;
    };
    MetadataStorage.prototype.ensureModelExists = function (modelName, klass) {
        if (!this.models[modelName]) {
            this.models[modelName] = {
                name: modelName,
                columns: [],
                klass: klass
                // endpoints: []
            };
        }
    };
    // addEndpont(modelName: string, endpointType: EndpointType) {
    //   this.ensureModelExists(modelName);
    // }
    MetadataStorage.prototype.addField = function (type, target, columnName, options) {
        if (options === void 0) { options = {}; }
        var modelName = target.name;
        if (!modelName) {
            return; // Not sure if this is needed anymore.  Previously this prevented base classes from registering
        }
        if (this.interfaces.indexOf(modelName) > -1) {
            return; // Don't add interfaces
        }
        this.ensureModelExists(modelName, target);
        this.models[modelName].columns.push(__assign({ type: type, propertyName: columnName }, options));
    };
    MetadataStorage.prototype.uniquesForModel = function (model) {
        return model.columns.filter(function (column) { return column.unique; }).map(function (column) { return column.propertyName; });
    };
    MetadataStorage.prototype.addInterfaceType = function (name) {
        this.addModel(name, null, '', { abstract: true });
    };
    MetadataStorage = __decorate([
        typedi_1.Service('MetadataStorage')
    ], MetadataStorage);
    return MetadataStorage;
}());
exports.MetadataStorage = MetadataStorage;
function getMetadataStorage() {
    if (!global.WarthogMetadataStorage) {
        // Since we can't use DI to inject this, just call into the container directly
        global.WarthogMetadataStorage = typedi_1.Container.get('MetadataStorage');
    }
    return global.WarthogMetadataStorage;
}
exports.getMetadataStorage = getMetadataStorage;
//# sourceMappingURL=metadata-storage.js.map