"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaGenerator = void 0;
var prettier = require("prettier");
var typedi_1 = require("typedi");
var typeorm_1 = require("typeorm");
var core_1 = require("../core");
var metadata_1 = require("../metadata");
var type_conversion_1 = require("./type-conversion");
var ignoreBaseModels = ['BaseModel', 'BaseModelUUID', 'IdModel'];
var SchemaGenerator = /** @class */ (function () {
    function SchemaGenerator(config) {
        this.config = config;
        this.metadata = metadata_1.getMetadataStorage();
    }
    SchemaGenerator.prototype.generate = function (
    // This will reference 'warthog in the deployed module, but we need to do a relative import in the examples library
    warthogImportPath) {
        var _this = this;
        if (warthogImportPath === void 0) { warthogImportPath = 'warthog'; }
        // Need to make sure we've collected all of the metadata off of our models
        this.metadata.collect();
        var templateArr = [
            "\n      // This file has been auto-generated by Warthog.  Do not update directly as it\n      // will be re-written.  If you need to change this file, update models or add\n      // new TypeGraphQL objects\n      // prettier-ignore\n      // @ts-ignore\n      import { DateResolver as Date } from 'graphql-scalars';\n      // prettier-ignore\n      // @ts-ignore\n      import { GraphQLID as ID } from 'graphql';\n      // prettier-ignore\n      // @ts-ignore\n      import { ArgsType, Field as TypeGraphQLField, Float, InputType as TypeGraphQLInputType, Int } from 'type-graphql';\n      // prettier-ignore\n      // @ts-ignore\n      import { registerEnumType, GraphQLISODateTime as DateTime } from \"type-graphql\";\n\n      // prettier-ignore\n      // @ts-ignore eslint-disable-next-line @typescript-eslint/no-var-requires\n      const { GraphQLJSONObject } = require('graphql-type-json');\n      // prettier-ignore\n      // @ts-ignore\n\n      import { BaseWhereInput, JsonObject, PaginationArgs, DateOnlyString, DateTimeString, IDType } from '" + warthogImportPath + "';\n      "
        ];
        templateArr.push.apply(templateArr, this.generateEnumMapImports());
        templateArr.push.apply(templateArr, this.generateClassImports());
        Object.keys(this.metadata.getModels()).forEach(function (modelName) {
            var model = _this.metadata.getModel(modelName);
            // If model is listed as dbOnly, don't generate any schema
            if (model.dbOnly === true) {
                return;
            }
            templateArr.push(_this.entityToOrderByEnum(model));
            templateArr.push(_this.entityToWhereInput(model));
            templateArr.push(_this.entityToWhereUniqueInput(model));
            templateArr.push(_this.entityToCreateInput(model));
            templateArr.push(_this.entityToUpdateInput(model));
            templateArr.push(_this.entityToWhereArgs(model));
            templateArr.push(_this.entityToCreateManyArgs(model));
            templateArr.push(_this.entityToUpdateInputArgs(model));
        });
        return this.format(templateArr.join('\n'));
    };
    SchemaGenerator.prototype.getColumnsForModel = function (model) {
        var models = [model];
        var columns = {};
        var superProto = model.klass ? model.klass.__proto__ : null;
        while (superProto) {
            var superModel = this.metadata.getModel(superProto.name);
            superModel && models.unshift(superModel);
            superProto = superProto.__proto__;
        }
        models.forEach(function (aModel) {
            aModel.columns.forEach(function (col) {
                columns[col.propertyName] = col;
            });
        });
        return Object.values(columns);
    };
    SchemaGenerator.prototype.filenameToImportPath = function (filename) {
        return filename.replace(/\.(j|t)s$/, '').replace(/\\/g, '/');
    };
    SchemaGenerator.prototype.columnToGraphQLDataType = function (column) {
        return type_conversion_1.columnTypeToGraphQLDataType(column.type, column.enumName);
    };
    SchemaGenerator.prototype.columnToTypeScriptType = function (column) {
        return type_conversion_1.columnInfoToTypeScriptType(column.type, column.enumName);
    };
    SchemaGenerator.prototype.generateEnumMapImports = function () {
        var _this = this;
        var imports = [];
        var enumMap = this.metadata.enumMap;
        // Keep track of already imported items so that we don't attempt to import twice in the event the
        // enum is used in multiple models
        var imported = new Set();
        Object.keys(enumMap).forEach(function (tableName) {
            Object.keys(enumMap[tableName]).forEach(function (columnName) {
                var enumColumn = enumMap[tableName][columnName];
                if (imported.has(enumColumn.name)) {
                    return;
                }
                imported.add(enumColumn.name);
                var filename = _this.filenameToImportPath(enumColumn.filename);
                imports.push("import { " + enumColumn.name + " } from '" + filename + "'\n  ");
            });
        });
        return imports;
    };
    SchemaGenerator.prototype.generateClassImports = function () {
        var _this = this;
        var imports = [];
        var classMap = this.metadata.classMap;
        Object.keys(classMap).forEach(function (tableName) {
            var classObj = classMap[tableName];
            var filename = _this.filenameToImportPath(classObj.filename);
            // Need to ts-ignore here for when we export compiled code
            // otherwise, it says we can't find a declaration file for this from the compiled code
            imports.push('// @ts-ignore\n');
            imports.push("import { " + classObj.name + " } from '" + filename + "'\n  ");
        });
        return imports;
    };
    SchemaGenerator.prototype.entityToWhereUniqueInput = function (model) {
        var _this = this;
        var uniques = this.metadata.uniquesForModel(model);
        var others = typeorm_1.getMetadataArgsStorage().uniques;
        var modelUniques = {};
        others.forEach(function (o) {
            var name = o.target.name;
            var columns = o.columns;
            if (name === model.name && columns) {
                columns.forEach(function (col) {
                    modelUniques[col] = col;
                });
            }
        });
        uniques.forEach(function (unique) {
            modelUniques[unique] = unique;
        });
        var distinctUniques = Object.keys(modelUniques);
        // If there is only one unique field, it should not be nullable
        var uniqueFieldsAreNullable = distinctUniques.length > 1;
        var fieldsTemplate = '';
        var modelColumns = this.getColumnsForModel(model);
        modelColumns.forEach(function (column) {
            // Uniques can be from Field or Unique annotations
            if (!modelUniques[column.propertyName]) {
                return;
            }
            var nullable = uniqueFieldsAreNullable ? ', { nullable: true }' : '';
            var graphQLDataType = _this.columnToGraphQLDataType(column);
            var tsType = _this.columnToTypeScriptType(column);
            if (column.array) {
                tsType = tsType.concat('[]');
                graphQLDataType = "[" + graphQLDataType + "]";
            }
            fieldsTemplate += "\n          @TypeGraphQLField(() => " + graphQLDataType + nullable + ")\n          " + column.propertyName + "?: " + tsType + ";\n        ";
        });
        if (!fieldsTemplate.length) {
            return ''; // If there are no uniques, don't generate the input
        }
        var superName = model.klass ? model.klass.__proto__.name : null;
        var classDeclaration = superName && !ignoreBaseModels.includes(superName)
            ? model.name + "WhereUniqueInput extends " + superName + "WhereUniqueInput"
            : model.name + "WhereUniqueInput";
        var template = "\n      @TypeGraphQLInputType()\n      export class " + classDeclaration + " {\n        " + fieldsTemplate + "\n      }\n    ";
        return template;
    };
    SchemaGenerator.prototype.entityToCreateInput = function (model) {
        var _this = this;
        var idsOnCreate = this.config.get('ALLOW_OPTIONAL_ID_ON_CREATE') === 'true';
        var fieldTemplates = '';
        if (idsOnCreate) {
            fieldTemplates += "\n        @TypeGraphQLField({ nullable: true })\n        id?: IDType; " + '' /* V3 breaking change: this was type string */ + "\n      ";
        }
        var modelColumns = this.getColumnsForModel(model);
        modelColumns.forEach(function (column) {
            if (column.readonly || column.computed) {
                return;
            }
            var graphQLDataType = _this.columnToGraphQLDataType(column);
            var nullable = column.nullable ? '{ nullable: true }' : '';
            var tsRequired = column.nullable ? '?' : '!';
            var tsType = _this.columnToTypeScriptType(column);
            if (column.array) {
                tsType = tsType.concat('[]');
                graphQLDataType = "[" + graphQLDataType + "]";
            }
            if (_this.columnRequiresExplicitGQLType(column)) {
                fieldTemplates += "\n            @TypeGraphQLField(() => " + graphQLDataType + ", " + nullable + ")\n            " + column.propertyName + tsRequired + ": " + tsType + ";\n         ";
            }
            else {
                fieldTemplates += "\n            @TypeGraphQLField(" + nullable + ")\n            " + column.propertyName + tsRequired + ": " + tsType + ";\n          ";
            }
        });
        var superName = model.klass ? model.klass.__proto__.name : null;
        var classDeclaration = superName && !ignoreBaseModels.includes(superName)
            ? model.name + "CreateInput extends " + superName + "CreateInput"
            : model.name + "CreateInput";
        return "\n      @TypeGraphQLInputType()\n      export class " + classDeclaration + " {\n        " + fieldTemplates + "\n      }\n    ";
    };
    SchemaGenerator.prototype.entityToUpdateInput = function (model) {
        var _this = this;
        var fieldTemplates = '';
        var modelColumns = this.getColumnsForModel(model);
        modelColumns.forEach(function (column) {
            if (column.readonly || column.computed) {
                return;
            }
            // TODO: also don't allow updated foreign key fields
            // Example: photo.userId: String
            var graphQLDataType = _this.columnToGraphQLDataType(column);
            var tsType = _this.columnToTypeScriptType(column);
            if (column.array) {
                tsType = tsType.concat('[]');
                graphQLDataType = "[" + graphQLDataType + "]";
            }
            if (_this.columnRequiresExplicitGQLType(column)) {
                fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n          " + column.propertyName + "?: " + tsType + ";\n        ";
            }
            else {
                fieldTemplates += "\n          @TypeGraphQLField({ nullable: true })\n          " + column.propertyName + "?: " + tsType + ";\n        ";
            }
        });
        var superName = model.klass ? model.klass.__proto__.name : null;
        var classDeclaration = superName && !ignoreBaseModels.includes(superName)
            ? model.name + "UpdateInput extends " + superName + "UpdateInput"
            : model.name + "UpdateInput";
        return "\n      @TypeGraphQLInputType()\n      export class " + classDeclaration + " {\n        " + fieldTemplates + "\n      }\n    ";
    };
    // Constructs required arguments needed when doing an update
    SchemaGenerator.prototype.entityToUpdateInputArgs = function (model) {
        return "\n      @ArgsType()\n      export class " + model.name + "UpdateArgs {\n        @TypeGraphQLField() data!: " + model.name + "UpdateInput;\n        @TypeGraphQLField() where!: " + model.name + "WhereUniqueInput;\n      }\n    ";
    };
    SchemaGenerator.prototype.columnToTypes = function (column) {
        var graphqlType = type_conversion_1.columnToGraphQLType(column.type, column.enumName);
        var tsType = this.columnToTypeScriptType(column);
        return { graphqlType: graphqlType, tsType: tsType };
    };
    SchemaGenerator.prototype.entityToWhereInput = function (model) {
        var _this = this;
        var filterByDefault = this.config.get('FILTER_BY_DEFAULT') === 'true';
        var fieldTemplates = '';
        function allowFilter(column, op) {
            if (column.filter === true) {
                return true;
            }
            else if (column.filter === false) {
                return false;
            }
            else if (Array.isArray(column.filter)) {
                return column.filter.includes(op);
            }
            else {
                return filterByDefault;
            }
        }
        var modelColumns = this.getColumnsForModel(model);
        modelColumns.forEach(function (column) {
            // If user specifically says not to filter (filter: false), then don't provide where inputs
            // Also, if the columns is "write only", then it cannot therefore be read and shouldn't have filters
            if (column.filter === false || column.writeonly === true) {
                return;
            }
            var tsType = _this.columnToTypes(column).tsType;
            var graphQLDataType = _this.columnToGraphQLDataType(column);
            // TODO: for foreign key fields, only allow the same filters as ID below
            // Example: photo.userId: String
            if (column.array) {
                fieldTemplates += "\n          @TypeGraphQLField(() => [" + graphQLDataType + "],{ nullable: true })\n          " + column.propertyName + "_containsAll?: [" + tsType + "];\n  \n          @TypeGraphQLField(() => [" + graphQLDataType + "],{ nullable: true })\n          " + column.propertyName + "_containsNone?: [" + tsType + "];\n  \n          @TypeGraphQLField(() => [" + graphQLDataType + "],{ nullable: true })\n          " + column.propertyName + "_containsAny?: [" + tsType + "];\n        ";
            }
            else if (column.type === 'id') {
                var graphQlType = 'ID';
                if (allowFilter(column, 'eq')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQlType + ",{ nullable: true })\n          " + column.propertyName + "_eq?: string;\n        ";
                }
                if (allowFilter(column, 'in')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => [" + graphQlType + "], { nullable: true })\n          " + column.propertyName + "_in?: string[];\n          ";
                }
            }
            else if (column.type === 'boolean') {
                if (allowFilter(column, 'eq')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ",{ nullable: true })\n          " + column.propertyName + "_eq?: Boolean;\n          ";
                }
                // V3: kill the boolean "in" clause
                if (allowFilter(column, 'in')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => [" + graphQLDataType + "], { nullable: true })\n          " + column.propertyName + "_in?: Boolean[];\n        ";
                }
            }
            else if (column.type === 'string' || column.type === 'email') {
                // TODO: do we need NOT?
                // `${column.propertyName}_not`
                if (allowFilter(column, 'eq')) {
                    fieldTemplates += "\n            @TypeGraphQLField({ nullable: true })\n            " + column.propertyName + "_eq?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'contains')) {
                    fieldTemplates += "\n            @TypeGraphQLField({ nullable: true })\n            " + column.propertyName + "_contains?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'startsWith')) {
                    fieldTemplates += "\n            @TypeGraphQLField({ nullable: true })\n            " + column.propertyName + "_startsWith?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'endsWith')) {
                    fieldTemplates += "\n            @TypeGraphQLField({ nullable: true })\n            " + column.propertyName + "_endsWith?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'in')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => [" + graphQLDataType + "], { nullable: true })\n            " + column.propertyName + "_in?: " + tsType + "[];\n        ";
                }
            }
            else if (column.type === 'float' ||
                column.type === 'integer' ||
                column.type === 'numeric') {
                if (allowFilter(column, 'eq')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n          " + column.propertyName + "_eq?: " + tsType + ";\n        ";
                }
                if (allowFilter(column, 'gt')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n          " + column.propertyName + "_gt?: " + tsType + ";\n        ";
                }
                if (allowFilter(column, 'gte')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n          " + column.propertyName + "_gte?: " + tsType + ";\n        ";
                }
                if (allowFilter(column, 'lt')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n          " + column.propertyName + "_lt?: " + tsType + ";\n        ";
                }
                if (allowFilter(column, 'lte')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n          " + column.propertyName + "_lte?: " + tsType + ";\n        ";
                }
                if (allowFilter(column, 'in')) {
                    fieldTemplates += "\n          @TypeGraphQLField(() => [" + graphQLDataType + "], { nullable: true })\n          " + column.propertyName + "_in?: " + tsType + "[];\n        ";
                }
            }
            else if (column.type === 'date' ||
                column.type === 'datetime' ||
                column.type === 'dateonly') {
                // I really don't like putting this magic here, but it has to go somewhere
                // This deletedAt_all turns off the default filtering of soft-deleted items
                if (column.propertyName === 'deletedAt') {
                    fieldTemplates += "\n          @TypeGraphQLField({ nullable: true })\n            deletedAt_all?: Boolean;\n          ";
                }
                if (allowFilter(column, 'eq')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n            " + column.propertyName + "_eq?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'lt')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n            " + column.propertyName + "_lt?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'lte')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n            " + column.propertyName + "_lte?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'gt')) {
                    fieldTemplates += "   \n            @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n            " + column.propertyName + "_gt?: " + tsType + ";\n          ";
                }
                if (allowFilter(column, 'gte')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n            " + column.propertyName + "_gte?: " + tsType + ";\n        ";
                }
            }
            else if (column.type === 'enum') {
                if (allowFilter(column, 'eq')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => " + graphQLDataType + ", { nullable: true })\n            " + column.propertyName + "_eq?: " + graphQLDataType + ";\n        ";
                }
                if (allowFilter(column, 'in')) {
                    fieldTemplates += "\n            @TypeGraphQLField(() => [" + graphQLDataType + "], { nullable: true })\n            " + column.propertyName + "_in?: " + graphQLDataType + "[];\n        ";
                }
            }
            else if (column.type === 'json') {
                fieldTemplates += "\n          @TypeGraphQLField(() => GraphQLJSONObject, { nullable: true })\n          " + column.propertyName + "_json?: JsonObject;\n        ";
            }
        });
        var superName = model.klass ? model.klass.__proto__.name : null;
        var classDeclaration = superName && !ignoreBaseModels.includes(superName)
            ? model.name + "WhereInput extends " + superName + "WhereInput"
            : model.name + "WhereInput";
        return "\n      @TypeGraphQLInputType()\n      export class " + classDeclaration + " {\n        " + fieldTemplates + "\n      }\n    ";
    };
    SchemaGenerator.prototype.entityToWhereArgs = function (model) {
        return "\n      @ArgsType()\n      export class " + model.name + "WhereArgs extends PaginationArgs {\n        @TypeGraphQLField(() => " + model.name + "WhereInput, { nullable: true })\n        where?: " + model.name + "WhereInput;\n  \n        @TypeGraphQLField(() => " + model.name + "OrderByEnum, { nullable: true })\n        orderBy?: " + model.name + "OrderByEnum;\n      }\n    ";
    };
    // Note: it would be great to inject a single `Arg` with the [model.nameCreateInput] array arg,
    // but that is not allowed by TypeGraphQL
    SchemaGenerator.prototype.entityToCreateManyArgs = function (model) {
        return "\n      @ArgsType()\n      export class " + model.name + "CreateManyArgs {\n        @TypeGraphQLField(() => [" + model.name + "CreateInput])\n        data!: " + model.name + "CreateInput[];\n      }\n    ";
    };
    SchemaGenerator.prototype.entityToOrderByEnum = function (model) {
        // user FILTER_BY_DEFAULT for both sort and filter
        var sortByDefault = this.config.get('FILTER_BY_DEFAULT') === 'true';
        var modelColumns = this.getColumnsForModel(model);
        var sortable = modelColumns
            .filter(function (column) {
            // Reasons we won't sort:
            // If user says this is not sortable
            // If column is type json
            // Also, if the column is "write only", therefore it cannot be read and shouldn't be sortable
            // Also, doesn't make sense to sort arrays
            if (column.sort === true) {
                return true;
            }
            if (column.sort === false ||
                column.type === 'json' ||
                column.type === 'text' ||
                column.writeonly ||
                column.array) {
                return false;
            }
            // If sorting hasn't been explicitly turned on or off, just use the default based on the ENV var
            return sortByDefault;
        })
            .map(function (column) { return column.propertyName; });
        // At a minimum, allow sorting by id if no sorts are specified
        if (!sortable.length) {
            sortable.push('id');
        }
        return "\n      export enum " + model.name + "OrderByEnum {\n        " + sortable
            .map(function (name) {
            return "\n            " + name + "_ASC = '" + name + "_ASC',\n            " + name + "_DESC = '" + name + "_DESC',\n          ";
        })
            .join('') + "\n      }\n  \n      registerEnumType(" + model.name + "OrderByEnum, {\n        name: '" + model.name + "OrderByInput'\n      });\n    ";
    };
    SchemaGenerator.prototype.columnRequiresExplicitGQLType = function (column) {
        return (column.enum ||
            column.array ||
            column.type === 'json' ||
            column.type === 'id' ||
            column.type === 'date' ||
            column.type === 'datetime' ||
            column.type === 'dateonly');
    };
    SchemaGenerator.prototype.format = function (code, options) {
        if (options === void 0) { options = {}; }
        try {
            // TODO: grab our prettier options (single quote, etc...)
            return prettier.format(code, __assign(__assign({}, options), { parser: 'typescript' }));
        }
        catch (e) {
            console.error("There is a syntax error in generated code, unformatted code printed, error: " + JSON.stringify(e));
            return code;
        }
    };
    SchemaGenerator = __decorate([
        typedi_1.Service('SchemaGenerator'),
        __param(0, typedi_1.Inject('Config')),
        __metadata("design:paramtypes", [core_1.Config])
    ], SchemaGenerator);
    return SchemaGenerator;
}());
exports.SchemaGenerator = SchemaGenerator;
//# sourceMappingURL=SchemaGenerator.js.map