import { DeepPartial, EntityManager, Repository, SelectQueryBuilder } from 'typeorm';
import { MetadataStorage } from '../metadata';
import { StandardDeleteResponse } from '../tgql';
import { ConnectionInputFields, GraphQLInfoService } from './GraphQLInfoService';
import { ConnectionResult, RelayPageOptions, RelayService } from './RelayService';
import { StringMap } from './types';
export interface BaseOptions {
    manager?: EntityManager;
}
export declare type BaseOptionsExtended = BaseOptions | EntityManager;
export interface Node {
    id?: string | number;
    getValue(field: string): string | number;
}
export declare type LimitOffset = {
    limit: number;
    offset?: number;
};
export declare type PaginationOptions = LimitOffset | RelayPageOptions;
export declare type RelayPageOptionsInput = {
    first?: number;
    after?: string;
    last?: number;
    before?: string;
};
export declare class BaseService<E extends Node> {
    protected entityClass: any;
    protected repository: Repository<E>;
    manager: EntityManager;
    columnMap: StringMap;
    klass: string;
    relayService: RelayService;
    graphQLInfoService: GraphQLInfoService;
    metadata: MetadataStorage;
    constructor(entityClass: any, repository: Repository<E>);
    hasColumn(name: string): boolean;
    getPageInfo(limit: number, offset: number, totalCount: number): {
        hasNextPage: boolean;
        hasPreviousPage: boolean;
        limit: number;
        offset: number;
        totalCount: number;
    };
    find(where?: any, // TODO: fix any
    orderBy?: string, limit?: number, offset?: number, fields?: string[], userId?: string, options?: BaseOptionsExtended): Promise<E[]>;
    findConnection(whereUserInput?: any, // V3: WhereExpression = {},
    orderBy?: string | string[], _pageOptions?: RelayPageOptionsInput, fields?: ConnectionInputFields, userId?: string, // Allow this param so that when we overload we can pass the userId for filtering data to user's "world"
    options?: BaseOptionsExtended): Promise<ConnectionResult<E>>;
    buildFindQuery<W>(where?: any, // W | WhereExpression
    orderBy?: string | string[], pageOptions?: LimitOffset, fields?: string[], userId?: string, // Allow this param so that when we overload we can pass the userId for filtering data to user's "world"
    options?: BaseOptionsExtended): SelectQueryBuilder<E>;
    findOne<W>(where: W, userId?: string, options?: BaseOptionsExtended): Promise<E>;
    extractManager(options?: BaseOptionsExtended): EntityManager;
    create(data: DeepPartial<E>, userId: string, options?: BaseOptionsExtended): Promise<E>;
    createMany(data: DeepPartial<E>[], userId: string, options?: BaseOptionsExtended): Promise<E[]>;
    update<W>(data: DeepPartial<E>, where: W, userId: string, options?: BaseOptionsExtended): Promise<E>;
    delete(where: any, userId: string, options?: BaseOptionsExtended): Promise<StandardDeleteResponse>;
    attrsToDBColumns: (attrs: string[]) => string[];
    attrToDBColumn: (attr: string) => string;
}
