import { GraphQLEnumType } from 'graphql';
import { ClassType } from '../core';
import { ColumnType, WhereOperator } from '../torm';
export declare type FieldType = 'boolean' | 'date' | 'dateonly' | 'datetime' | 'email' | 'enum' | 'float' | 'id' | 'integer' | 'json' | 'numeric' | 'string' | 'text';
export interface DecoratorCommonOptions {
    apiOnly?: boolean;
    computed?: boolean;
    dbOnly?: boolean;
    description?: string;
    editable?: boolean;
    filter?: boolean | WhereOperator[];
    nullable?: boolean;
    readonly?: boolean;
    sort?: boolean;
    writeonly?: boolean;
}
export interface ColumnMetadata extends DecoratorCommonOptions {
    type: FieldType;
    propertyName: string;
    dataType?: ColumnType;
    default?: any;
    enum?: GraphQLEnumType;
    enumName?: string;
    specialType?: 'primary' | 'created-at' | 'created-by' | 'updated-at' | 'updated-by' | 'deleted-at' | 'deleted-by' | 'owner' | 'version';
    unique?: boolean;
    array?: boolean;
}
export declare type ColumnOptions = Partial<ColumnMetadata>;
export interface ModelMetadata {
    abstract?: boolean;
    filename?: string;
    klass?: any;
    name: string;
    columns: ColumnMetadata[];
    apiOnly?: boolean;
    dbOnly?: boolean;
}
export declare class MetadataStorage {
    enumMap: {
        [table: string]: {
            [column: string]: any;
        };
    };
    classMap: {
        [table: string]: any;
    };
    models: {
        [table: string]: ModelMetadata;
    };
    interfaces: string[];
    addModel(name: string, klass: any, filename: string, options?: Partial<ModelMetadata>): void;
    addEnum(target: ClassType, columnName: string, enumName: string, enumValues: any, filename: string, options: ColumnOptions): void;
    getModels(): {
        [table: string]: ModelMetadata;
    };
    collect(): this;
    getModel(name: string): ModelMetadata;
    getModelColumns(modelName: string): string[];
    getEnum(modelName: string, columnName: string): any;
    ensureModelExists(modelName: string, klass: ClassType): void;
    addField(type: FieldType, target: ClassType, columnName: string, options?: Partial<ColumnMetadata>): void;
    uniquesForModel(model: ModelMetadata): string[];
    addInterfaceType(name: string): void;
}
export declare function getMetadataStorage(): MetadataStorage;
